@@ -6,8 +6,8 @@
 SPVM_MY* SPVM_MY_new(SPVM_COMPILER* compiler) {
   SPVM_MY* my = SPVM_COMPILER_ALLOCATOR_safe_malloc_zero(compiler, sizeof(SPVM_MY));
 
-  my->index = -1;
-  my->var_id = -1;
+  my->id = -1;
+  my->mem_id = -1;
   
   return my;
 }
